@@ -1,4 +1,5 @@
 from __future__ import annotations
+from os import error
 
 from typing import List, Optional, Tuple
 
@@ -8,12 +9,16 @@ from parchmint.target import Target
 
 
 class ConnectionPath:
+    """Describes the connection path which is a member of the connection, it consists of
+    the source and sink targets and a list of waypoints that connect them.
+    """
+
     def __init__(
         self,
         source: Target = None,
         sink: Target = None,
         waypoints: List[Tuple[int, int]] = None,
-        json=None,
+        json_data=None,
     ) -> None:
         """Creates a new connection path object
 
@@ -29,27 +34,53 @@ class ConnectionPath:
         self.__sink: Optional[Target] = sink
         self.__waypoints: List[Tuple[int, int]] = waypoints
 
-        if json is not None:
-            self.parse_parchmint_v1(json)
+        if json_data is not None:
+            self.parse_parchmint_v1(json_data)
 
     @property
     def source(self) -> Target:
+        """Returns the source information of the connection path
+
+        Raises:
+            Error: No source was set in the connection path
+
+        Returns:
+            Target: Source information
+        """
         if self.__source is None:
-            raise AssertionError
+            raise error("No value set to the source")
         return self.__source
 
     @property
     def sink(self) -> Target:
+        """Returns the sink information of the connection path
+
+        Raises:
+            Error: No sink was set in the connection path
+
+        Returns:
+            Target: Sink information
+        """
         if self.__sink is None:
-            raise AssertionError
+            raise error("No value set to the sink")
         return self.__sink
 
     @property
     def waypoints(self) -> List[Tuple[int, int]]:
+        """Returns the waypoints in the connection path
+
+        Returns:
+            List[Tuple[int, int]]: The list of waypoints
+        """
         return self.__waypoints
 
     @waypoints.setter
     def waypoints(self, value: List[Tuple[int, int]]):
+        """Sets teh waypoints of the connection path
+
+        Args:
+            value (List[Tuple[int, int]]): List of co-ordinate tuples
+        """
         self.__waypoints = value
 
     def to_parchmint_v1(self):
@@ -66,14 +97,25 @@ class ConnectionPath:
             "wayPoints": [list(wp) for wp in self.__waypoints],
         }
 
-    def parse_parchmint_v1(self, json) -> None:
-        self.__source = Target(json=json["source"])
-        self.__sink = Target(json=json["sink"])
-        self.__waypoints = [(wp[0], wp[1]) for wp in json["wayPoints"]]
+    def parse_parchmint_v1(self, json_data) -> None:
+        self.__source = Target(json=json_data["source"])
+        self.__sink = Target(json=json_data["sink"])
+        self.__waypoints = (
+            [(wp[0], wp[1]) for wp in json_data["wayPoints"]]
+            if json_data["wayPoints"] is not None
+            else []
+        )
 
 
 class Connection:
-    def __init__(self, json=None, device_ref=None):
+    """Connection Object represented in parchmint
+
+    Connection object encapsulates all types of channels that can be drawn
+    to connect different microfluidic components.
+
+    """
+
+    def __init__(self, json_data=None, device_ref=None):
         """[summary]
 
         Args:
@@ -91,14 +133,15 @@ class Connection:
         self.sinks: List[Target] = []
         self.layer: Optional[Layer] = None
         self._paths: List[ConnectionPath] = []
+        self.features: List[str] = []
 
-        if json:
+        if json_data:
             if device_ref is None:
                 raise Exception(
                     "Cannot Parse Connection from JSON with no Device Reference, check device_ref parameter in constructor "
                 )
 
-            self.parse_from_json(json, device_ref)
+            self.parse_from_json_v1_x(json_data, device_ref)
 
     def parse_from_json(self, json, device_ref=None):
         """Parses from the json dict
@@ -124,21 +167,47 @@ class Connection:
         self.params = Params(json["params"])
 
         self.source = Target(json["source"])
-
         if "sinks" in json.keys():
-            if json["sinks"]:
-                for target in json["sinks"]:
+            for target in json["sinks"]:
+                self.sinks.append(Target(target))
+
+    def parse_from_json_v1_x(self, json_data, device_ref=None):
+        """Parses from the json dict - v1_x
+
+        Args:
+            json (dict): json dict after json.loads()
+        """
+        if device_ref is None:
+            raise Exception(
+                "Cannot Parse Connection from JSON with no Device Reference, check device_ref parameter in constructor "
+            )
+        self.name = json_data["name"]
+        self.ID = json_data["id"]
+        self.layer = device_ref.get_layer(json_data["layer"])
+
+        # Pull out the paths
+        if "paths" in json_data.keys():
+            json_paths = json_data["paths"]
+            for json_path in json_paths:
+                self._paths.append(ConnectionPath(json_data=json_path))
+        else:
+            print("No path data found for connection {}".format(self.ID))
+
+        self.params = Params(json_data["params"])
+
+        self.source = Target(json_data["source"])
+
+        if "sinks" in json_data.keys():
+            if json_data["sinks"]:
+                for target in json_data["sinks"]:
                     self.sinks.append(Target(target))
             else:
                 print("connection", self.name, "does not have any sinks")
         else:
             print("connection", self.name, "does not have any sinks")
 
-        # TODO - Change this in the v1.2 version
-        if "waypoints" in json.keys():
-            waypoints_raw = json["waypoints"]
-            waypoints = [(wp[0], wp[1]) for wp in waypoints_raw]
-            self.add_waypoints_path(None, None, waypoints)
+        if "features" in json_data.keys():
+            self.features = json_data["features"]
 
     def __str__(self):
         return str(self.__dict__)
@@ -173,11 +242,34 @@ class Connection:
             "sinks": [s.to_parchmint_v1() for s in self.sinks],
             "name": self.name,
             "id": self.ID,
-            "source": self.source.to_parchmint_v1(),
-            "params": self.params.to_parchmint_v1(),
-            "layer": self.layer.ID,
+            "source": self.source.to_parchmint_v1() if self.source else None,
+            "params": self.params.to_parchmint_v1() if self.params else None,
+            "layer": self.layer.ID if self.layer else None,
         }
 
         ret["paths"] = [path.to_parchmint_v1() for path in self._paths]
 
         return ret
+
+    def to_parchmint_v1_x(self):
+        """Returns the updated json dict
+
+        Returns:
+            dict: dictionary that can be used in json.dumps()
+        """
+
+        ret = {
+            "sinks": [s.to_parchmint_v1() for s in self.sinks],
+            "name": self.name,
+            "id": self.ID,
+            "source": self.source.to_parchmint_v1() if self.source else None,
+            "params": self.params.to_parchmint_v1(),
+            "layer": self.layer.ID if self.layer else None,
+            "paths": [path.to_parchmint_v1() for path in self._paths],
+            "features": self.features,
+        }
+
+        return ret
+
+    def __hash__(self) -> int:
+        return hash(repr(self))
